Index: src/mips/lithium-codegen-mips.cc |
diff --git a/src/mips/lithium-codegen-mips.cc b/src/mips/lithium-codegen-mips.cc |
index 38b0ff5274d4c44cda241072aa8e7a7ab05a54be..50f58125a5039ee80cf48ab9747b4aa57096307a 100644 |
--- a/src/mips/lithium-codegen-mips.cc |
+++ b/src/mips/lithium-codegen-mips.cc |
@@ -856,26 +856,31 @@ void LCodeGen::DeoptimizeIf(Condition condition, |
void LCodeGen::RegisterDependentCodeForEmbeddedMaps(Handle<Code> code) { |
ZoneList<Handle<Map> > maps(1, zone()); |
+ ZoneList<Handle<JSObject> > objects(1, zone()); |
int mode_mask = RelocInfo::ModeMask(RelocInfo::EMBEDDED_OBJECT); |
for (RelocIterator it(*code, mode_mask); !it.done(); it.next()) { |
- RelocInfo::Mode mode = it.rinfo()->rmode(); |
- if (mode == RelocInfo::EMBEDDED_OBJECT && |
- it.rinfo()->target_object()->IsMap()) { |
- Handle<Map> map(Map::cast(it.rinfo()->target_object())); |
- if (map->CanTransition()) { |
+ if (Code::IsWeakEmbeddedObject(code->kind(), it.rinfo()->target_object())) { |
+ if (it.rinfo()->target_object()->IsMap()) { |
+ Handle<Map> map(Map::cast(it.rinfo()->target_object())); |
maps.Add(map, zone()); |
+ } else if (it.rinfo()->target_object()->IsJSObject()) { |
+ Handle<JSObject> object(JSObject::cast(it.rinfo()->target_object())); |
+ objects.Add(object, zone()); |
} |
} |
} |
#ifdef VERIFY_HEAP |
// This disables verification of weak embedded maps after full GC. |
// AddDependentCode can cause a GC, which would observe the state where |
// this code is not yet in the depended code lists of the embedded maps. |
- NoWeakEmbeddedMapsVerificationScope disable_verification_of_embedded_maps; |
+ NoWeakObjectVerificationScope disable_verification_of_embedded_objects; |
#endif |
for (int i = 0; i < maps.length(); i++) { |
maps.at(i)->AddDependentCode(DependentCode::kWeaklyEmbeddedGroup, code); |
} |
+ for (int i = 0; i < objects.length(); i++) { |
+ AddWeakObjectToCodeDependency(isolate()->heap(), objects.at(i), code); |
+ } |
} |
@@ -856,26 +856,31 @@ void LCodeGen::DeoptimizeIf(Condition condition, |
} |
#ifdef VERIFY_HEAP |
- // This disables verification of weak embedded maps after full GC. |
+ // This disables verification of weak embedded objects after full GC. |
// AddDependentCode can cause a GC, which would observe the state where |
// this code is not yet in the depended code lists of the embedded maps. |